@@ -32,6 +32,7 @@ outside of this lifecycle, the send_to_server function can be used to send and
 receive information from the server outside of the default times. Such
 communications should be limited to the body of an on_interact method.
 """
+BACKUP_FILE = ".ok_messages"
 
 from client import config
 from client.models import *
@@ -39,24 +40,27 @@ from client.protocols import *
 from client.utils import auth
 from client.utils import loading
 from client.utils import output
+from datetime import datetime
 from urllib import request, error
 import client
 import argparse
 import base64
 import json
 import multiprocessing
+import pickle
 import sys
 import time
 
 def send_to_server(access_token, messages, name, server,
-                   endpoint='submission'):
+                   insecure=False):
     """Send messages to server, along with user authentication."""
     data = {
         'assignment': name,
         'messages': messages,
     }
     try:
-        address = 'https://' + server + '/api/v1/' + endpoint
+        prefix = "http" if insecure else "https"
+        address = prefix + '://' + server + '/api/v1/submission'
         serialized = json.dumps(data).encode(encoding='utf-8')
         # TODO(denero) Wrap in timeout (maybe use PR #51 timed execution).
         # TODO(denero) Send access token with the request
@@ -69,9 +73,10 @@ def send_to_server(access_token, messages, name, server,
     except error.HTTPError as ex:
         # print("Error while sending to server: {}".format(ex))
         try:
-            #response_json = json.loads(response)
             if ex.code == 403:
-                get_latest_version(server)
+                response = ex.read().decode('utf-8')
+                response_json = json.loads(response)
+                get_latest_version(response_json['data']['download_link'])
             #message = response_json['message']
             #indented = '\n'.join('\t' + line for line in message.split('\n'))
             #print(indented)
@@ -81,30 +86,35 @@ def send_to_server(access_token, messages, name, server,
             # print("Couldn't connect to server")
             pass
 
+def dump_to_server(access_token, msg_queue, name, server, insecure, staging_queue):
+    while not msg_queue.empty():
+        message = msg_queue.get()
+        staging_queue.put(message)
+        try:
+            if send_to_server(access_token, message, name, server, insecure) == None:
+                staging_queue.get() #throw away successful message
+        except error.URLError as ex:
+            pass
+    return
+
 #####################
 # Software Updating #
 #####################
 
-def get_latest_version(server):
+def get_latest_version(download_link):
     """Check for the latest version of ok and update this file accordingly.
     """
     #print("We detected that you are running an old version of ok.py: {0}".format(VERSION))
 
     # Get server version
-    address = "https://" + server + "/api/v1" + "/version?name=okpy"
 
     try:
-        #print("Updating now...")
-        req = request.Request(address)
+        req = request.Request(download_link)
         response = request.urlopen(req)
 
-        full_response = json.loads(response.read().decode('utf-8'))
-
-        contents = base64.b64decode(
-            full_response['data']['results'][0]['file_data'])
-        new_file = open('ok', 'wb')
-        new_file.write(contents)
-        new_file.close()
+        zip_binary = response.read()
+        with open('ok', 'wb') as f:
+            f.write(zip_binary)
         #print("Done updating!")
     except error.HTTPError:
         # print("Error when downloading new version")
@@ -130,6 +140,8 @@ def parse_input():
                         help="unlock tests interactively")
     parser.add_argument('-v', '--verbose', action='store_true',
                         help="print more output")
+    parser.add_argument('--insecure', action='store_true',
+                        help="uses http instead of https")
     parser.add_argument('-i', '--interactive', action='store_true',
                         help="toggle interactive mode")
     parser.add_argument('-l', '--lock', type=str,
@@ -175,25 +187,47 @@ def main():
                      for p in protocol.get_protocols(config.protocols)]
 
         messages = dict()
+        msg_queue = multiprocessing.Queue()
+        file_contents = []
+
+        try:
+            with open(BACKUP_FILE, 'rb') as fp:
+                file_contents = pickle.load(fp)
+        except FileNotFoundError as e:
+            # File doesn't exist, so file_contents should be empty
+            pass
+        except EOFError as e:
+            # File is empty, so no messages are inside
+            pass
+
+        for message in file_contents:
+            msg_queue.put(message)
 
         for proto in protocols:
             messages[proto.name] = proto.on_start()
+        messages['timestamp'] = str(datetime.now())
 
         if not args.local:
             try:
                 access_token = auth.authenticate(args.authenticate)
+                msg_queue.put(messages)
+                staging_queue = multiprocessing.Queue()
                 server_thread = multiprocessing.Process(
-                    target=send_to_server,
-                    args=(access_token, messages, assignment['name'],
-                          args.server))
+                    target=dump_to_server,
+                    args=(access_token, msg_queue, assignment['name'],
+                          args.server, args.insecure, staging_queue))
                 server_thread.start()
             except error.URLError as ex:
                 # TODO(soumya) Make a better error message
                 # print("Nothing was sent to the server!")
                 pass
+        
+        interact_msg = {}
 
         for proto in protocols:
-            proto.on_interact()
+            interact_msg[proto.name] = proto.on_interact()
+
+        interact_msg['timestamp'] = str(datetime.now())
 
         # TODO(denero) Print server responses.
 
@@ -202,11 +236,23 @@ def main():
         loading.dump_tests(args.tests, assignment)
 
         if not args.local:
+            msg_queue.put(interact_msg)
+            
             while timer_thread.is_alive():
                 pass
 
             if not args.force:
                 server_thread.terminate()
+            else:
+                server_thread.join()
+
+            dump_list = []
+            while not msg_queue.empty():
+                dump_list.append(msg_queue.get_nowait())
+            while not staging_queue.empty():
+                dump_list.append(staging_queue.get_nowait())
+            with open(BACKUP_FILE, 'wb') as fp:
+                pickle.dump(dump_list, fp)
 
     except KeyboardInterrupt:
         if timer_thread:
